package com.example.teacher.controller;


import com.example.teacher.model.Teacher;
import io.restassured.RestAssured;
import io.restassured.http.ContentType;
import io.restassured.response.Response;
import lombok.extern.slf4j.Slf4j;
//import org.json.simple.JSONObject;
import org.hamcrest.Matcher;
import org.hamcrest.Matchers;
import org.junit.Assert;
import org.junit.Test;
import org.springframework.boot.test.autoconfigure.web.servlet.WebMvcTest;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;

import static org.hamcrest.Matchers.equalTo;
import static org.hamcrest.Matchers.samePropertyValuesAs;

@Slf4j
@WebMvcTest
public class TeacherEndToEndTest {

    @Test
    public void getAllTeachersStatusTest(){
        RestAssured
                .get("http://localhost:8081/teachers/all")
                .then()
                .assertThat()
                .statusCode(200)
                .body("[2].id", Matchers.equalTo(5))
                .body("[2].name", Matchers.equalTo("Mr Naveed"))
                .body("[2].age", Matchers.equalTo(25))
                .body("[2].subject", Matchers.equalTo("Math"))
                .body("[2].dateOfJoining", Matchers.equalTo("10:06:2008"))
                .log()
                .body();
    }

    @Test
    public void getTeacherByIdReturnsObjectTest() throws ParseException {
        Teacher teacher = RestAssured
                .given()
                .get("http://localhost:8081/teachers/32")
                .as(Teacher.class);

        Date date = new SimpleDateFormat("dd:MM:yyyy hh:mm:ss").parse("10:06:2008 06:00:00");
        Teacher myTeacher = new Teacher(32, "Naveed", 25, "28", date, true);
        Assert.assertThat(myTeacher, samePropertyValuesAs(teacher));
    }

    @Test
    public void getTeacherByWrongIdStatusTest() {
        RestAssured
                .given()
                .get("http://localhost:8081/teachers/411")
                .then()
                .statusCode(404)
                .log()
                .headers();
    }

    @Test
    public void getTeacherByIdTest() throws ParseException {
        Teacher myTeacher = new Teacher(1, "sahal", 25, "English", new Date(), true);

        Response response = RestAssured
                .get("http://localhost:8081/teachers/1")
                .then()
                .statusCode(302)
                .assertThat()
                .body("id", equalTo(myTeacher.getId()))
                .body("name", equalTo(myTeacher.getName()))
                .body("age", equalTo(myTeacher.getAge()))
                .body("dateOfJoining", equalTo("11:01:2003"))
                .body("subject", equalTo(myTeacher.getSubject()))
                .extract()
                .response();
        log.info(response.asPrettyString());
    }

    @Test
    public void getTeacherByWrongIdTest() {
        Response response = RestAssured
                .get("http://localhost:8081/teachers/999")
                .then()
                .statusCode(404)
                .extract().response();

        Assert.assertEquals(response.asString(), "Teacher does not exist");
        log.info(response.asPrettyString());
    }
//
//    @Test
//    public void createTeacherTest() throws JSONException, ParseException {
//        Date date = new Date();
//        SimpleDateFormat simpleDateFormat = new SimpleDateFormat("dd:MM:yyyy");
//        String stringDate = simpleDateFormat.format(date);
//
//        JSONObject requestParams = new JSONObject();
//        requestParams.put("name", "QadeerTest");
//        requestParams.put("age", "57");
//        requestParams.put("subject", "Math");
//        requestParams.put("dateOfJoining", stringDate);
//        RestAssured
//                .given()
//                .contentType(ContentType.JSON)
//                .body(requestParams.toString())
//                .when()
//                .post("http://localhost:8081/teachers")
//                .then()
//                .statusCode(201);
//    }

//    @Test
//    public void deleteTeacherByIdTest() {
//        RestAssured
//                .given()
//                .delete("http://localhost:8081/teachers/delete/36")
//                .then()
//                .statusCode(200);
//    }

    @Test
    public void deleteTeacherByWrongIdTest() {
        RestAssured
                .given()
                .delete("http://localhost:8081/teachers/delete/36")
                .then()
                .statusCode(404);
    }

    @Test
    public void updateTeacher() throws ParseException {
        Date date = new SimpleDateFormat("dd:MM:yyyy").parse("11:11:2011");
        Teacher teacher = new Teacher("TestTeacher", 25, "TestMaths", date);
        RestAssured
                .given()
                .body(teacher)
                .contentType(ContentType.JSON)
                .put("http://localhost:8081/teachers/35")
                .then()
                .statusCode(202)
                .log()
                .all();
    }

    @Test
    public void createTeacherWithObject() throws ParseException {
        Date date = new SimpleDateFormat("dd:MM:yyyy").parse("11:11:2011");
        Teacher teacher = new Teacher("TestTeacher", 25, "TestMaths", date);
        RestAssured
                .given()
                .contentType(ContentType.JSON)
                .body(teacher)
                .when()
                .post("http://localhost:8081/teachers")
                .then()
                .assertThat()
                .statusCode(201)
                .log()
                .body();
    }
}
